<?php
////////////////////////////////////////////////////////////////////////
/**
 *	the Context Object is the core object for the Pomo website management system. 
 *	It provides central functions and data access to all parts of the system.  
 *
 * @link        http://pomo.sourceforge.net for the latest version
 * @author	    Jason Suwala <jsuwala[at]gmail.com>
 * @copyright	Jason Suwala <jsuwala[at]gmail.com> - distributed under the GPL
 * 
 * @package Pomo_Website_Management_System
 * @version 0.1
 * @license http://www.opensource.org/licenses/gpl-license.php General Public License (GPL)
 * 
 */
////////////////////////////////////////////////////////////////////////
class Context {

	public $currentPage; // Page object
	public $isValid;	// boolean
	public $ViewMode;	// string
	public $currentPipeline; // an object
	public $currentUser; // an object
	public $db; // database object

	private $attachCache; // array
	private $pageCache; // array

	/**
	 * create a new Context object
	 * 
	 * @param string $passedparams the full path
	 * 	By going to the page "http://localhost/root/page.htm" passedparams is 'path=root/page.htm'
	 *	                      "http://localhost/"              passedparams is ''
	 *	                      "http://localhost/page.htm"      passedparams is 'path=page.htm'
	 *	                      "http://localhost/1/2/3/4.htm"   passedparams is 'path=1/2/3/4.htm'
	 *
	 * @todo add the option of adding the hostname to the passedparams list. This will allow multiple sites to be accessed using this one installation (similar to MSCMS' "Map Channel Names to Host Header Names Feature" http://support.microsoft.com/?id=326783)	
	 */
	public function Context ($passedparams) 
	{
		// a context is always created first, so start a session, and start output buffering.
		session_start();
		ob_start("ob_gzhandler");

		$this->isValid = true;
		
		$this->db = new DB();

		$this->currentPage = new Page($this);

		$this->pageCache = array();
		$this->attachCache = array();

		// remove the "path=" string
		$passedparams = substr($passedparams, 5);

		#####
		# start getPageByPath
		
		$pageOrFolder = $this->getPageByPath($passedparams);
		if($pageOrFolder === NULL) 
		{
			$this->isValid = false;
			OutputError(404);
			return;
		}

		
		if(is_page($pageOrFolder)) 
		{
			$this->currentPage = $pageOrFolder;	
			// if we are on a page, we can have a pipeline.
			 if ($this->isValid) 
			 {				 
				 $this->InitCurrentPipeline();    
			 }			
		}

		// load up the current User
		$this->currentUser = new User($this);
		$this->currentUser->LoadAndLoginFromSession();

		// set viewmode after currentuser has been loaded
		$this->ViewMode = $this->getViewMode(); // "edit" or "view"
			 
	} // constructor
	
	
	/**
	 * Initializes the current pipeline object (should only be done when the context is created)
	 * It reads the "pipeline" option for the current page, or tries to inherit the property
	 * from the parent page. 
	 * 
	 * Will output a 601 error if no pipeline can be found
	 */
	private function InitCurrentPipeline()
	{
		
		$this->currentPipeline = NULL;
		if (! $this->isValid)
		{
			return;
		}
		if ($this->currentPage->pipeline != null)
		{
			$this->currentPipeline = $this->currentPage->pipeline;
			return;
		}
		else
		{ 
			// currentpage->pipeline isn't set, so try getting it from the parent page			
			$page = $this->currentPage;			
			while($page != NULL) 
			{				
				if ($page->pipeline != null)
				{
					$this->currentPipeline = $page->pipeline;
					return;
				}				
				$page = $page->parentPage();	
								
			} // while
		} // else

		if ($this->currentPipeline == NULL)
		{
			$this->isValid = FALSE;
			OutputError("601");
		}	
	} // InitCurrentPipeline


	/**
	 * gets a page object by a given page ID (AKA a page GUID)
	 * 
	 * @param  paramname description
	 */
	public function getPageByPageID($_pageID) 
	{
		if($_pageID == "") 
		{
			debug("error: context->getPageByPageID :pageID is NULL");
		}

		if(! isset($this->pageCache[$_pageID]) ) 
		{
			# echo("<br>in getPageByPageID ".$_pageID);
			$page = new Page($this);
			$ret = $page->InitFromDatabaseByID($_pageID);
			if($ret === false) 
			{
				return NULL;	
			}
			else
			{
				$this->pageCache[$_pageID] = $page;
				return $page;
			}			
		}	
		else
		{
			return $this->pageCache[$_pageID];
		}
		
	} // getPageByPageID

	/**
	 * gets a Page based on a given path
	 * 
	 * @param string $_path the slash ('/') delimited path of the page to return. The path can have a query string and a filename extension - both of which will be ignored. 
	 * @return Page returns the page associated with a given path, or NULL if no page is found
	 */
	// returns NULL if path not found
	public function getPageByPath($_path) 
	{
		// add a '/' to start if needed
		if(strlen($_path) > 0 && $_path[0] != "/") 
		{
			$_path = "/".$_path;	
		}

		// take off '/' at end if it exists
		if(strEndsWith($_path,'/')) 
		{
			$_path = substr($_path,0, strlen($_path)-1); // take off last character					
		}
		
		// when URL = 'http://localhost/admin/createPage.htm?f=/root/', $_path = '/admin/createPage.htm&f=/root/'
		
		$pathparts1 = split("[&]",$_path);
		# debug($pathparts1);
		// $pathparts1[0] now has the page/folder requested; $pathparts1[1] has the querystring

		$pathparts = split ("[/]", $pathparts1[0]); 
		// check if there is enough information to execute on
		if(count($pathparts) < 1) 
		{			
			return NULL;
		}


		// when url = "http://localhost/root/", $_path = "/root/"
		//    --> when this is split, we get [0]=> 'root', [1]=>'' (note [1] is blank);
		// when url = "http://localhost/admin/createPage.htm?f=/root/", $_path = "/admin/createPage.htm&f=/root/"
		//    --> when this is split, we get [0]=> 'root', [1]=>'' (note [1] is blank);

		if($pathparts[count($pathparts)-1] == "")
		{
			array_pop($pathparts); // take the last item off.
		}

		if(count($pathparts) >= 3 && $pathparts[0] == "" && $pathparts[1] == "" )
		{
			// sometimes there may be too many blanks at the beginning - this gets rid of them
			array_shift($pathparts);
		}


		// get the actual page requested, seperated from the query string
		$pp_index = count($pathparts)-1;
		if($pp_index == -1) 
		{
			$pp_split = "";
		}
		else
		{
			$pp_split = $pathparts[$pp_index];
		}
		$fileparts = split("[?&]",$pp_split);		
		// $fileparts[0] now has the filename, [1] has the querystring		
		
		$PageName_withExtension = $fileparts[0];

		// remove the filename extension for the page being requested
		$PageName_noExtension = removeFileExtension($fileparts[0]);				
			
		// loop through everything except the last page
		$end_index = count($pathparts)-2; 
				 
		// go through each page and make sure that it exists. Cache the page data as we go through it. 
		$lastPageId = -1;
		$folder = NULL;
		for($i=0; $i <= $end_index; $i++)
		{
			
			$page = new Page($this);
						
			$ret = $page->InitFromDatabase($lastPageId, $pathparts[$i]); // params: parentpage, name
			if($ret === false) 
			{								

				debug("getPageByPath:page->init error: (should actually return NULL here) : lastID: ".$lastPageId.";  Loop: ".$i."; Name:'".$pathparts[$i]."'");
				return NULL;				
			}
			// let's cache the folder so that we can save the database hits (for this session)
			$this->pageCache[$page->GUID] = $page;

			$lastPageId = $page->GUID;
		} // for

		// init the last page
		$page = new Page($this);
		$page_ret = $page->InitFromDatabase($lastPageId, $PageName_noExtension );

		if($page_ret === false) 
		{
			return NULL;	
		}
		else
		{
			$this->pageCache[$page->GUID] = $page;
			return $page;
		}
		
		return NULL;

	} // getPageByPath

	public function getAttachmentByAttachmentID($_attachID) 
	{
		if($_attachID == "") 
		{
			debug("error: context->getAttachmentByAttachmentID :attachID is NULL");
		}

		if(! isset($this->attachCache[$_attachID]) ) 
		{
			# echo("<br>in getPageByPageID ".$_pageID);
			$att = new Attachment($this);
			
			$ret = $att->InitFromDatabaseByID($_attachID);
			
			if($ret === false) 
			{
				return NULL;	
			}
			else
			{
				$this->attachCache[$_attachID] = $att;
				return $att;
			}			
		}	
		else
		{
			return $this->attachCache[$_attachID];
		}
		
	} // getAttachmentByAttachmentID



	/**
	 * gets the current View Mode descriptor for the current user.
	 * 
	 * @return string returns either "edit" or "view"
	 * 
	 * @todo getViewMode() should probably be moved to the User class.
	 */
	private function getViewMode() 
	{
		// if not logged in, we are in view mode only
		if(! $this->currentUser->loggedIn) 
		{
			return "view";			
		}

		if(isset($_SESSION['viewmode']) && ($_SESSION['viewmode'] == "view" || $_SESSION['viewmode'] == "edit")) 
		{			
			return $_SESSION['viewmode'];			
		}
				

		return "view";		
		
	} // getViewMode
	
} // context Class

?>
